/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_LOWLEVEL_MORPHOLOGY_NEIGHBOR_OPERATORS_HPP__
#define YAYI_LOWLEVEL_MORPHOLOGY_NEIGHBOR_OPERATORS_HPP__

/*!@file
 * This file defines some neighbor operators
 */
 
#include <boost/circular_buffer.hpp>
#include <algorithm>

namespace yayi { namespace llmm {

  namespace neighborhood_operator_traits {
    struct shiftable_operator;
  }


  /*!@brief Min element over a range
   * The functor returns a reference to the min element over the provided range
   * @author Raffi Enficiaud
   */
  template <class in_t>
  struct s_min_element_subset {
  
    typedef neighborhood_operator_traits::shiftable_operator operator_t;
  
    template <class it>
    const in_t& operator()(it beg, const it e) const {
      DEBUG_ASSERT(beg != e, "Empty neighborhood");
      return *std::min_element(beg, e);
    }
    
    #if 0
    // for future use
    template <class it>
    struct s_shifter {
      typedef it iterator_t;
      typedef typename iterator_t::value_type value_type;
      
      //std::vector<value_type> vect_values;
      boost::circular_buffer<value_type> values;

      
      template <class neigh_t>
      s_shifter(const neigh_t& n) : values(n.get_shift()) {
      
      }
      
      // Initialize the internals
      void init() {
        values.clear();
      }
      
      // 
      const in_t& step(it beg, const it e) {
        values.push_back(*std::min_element(beg, e));
        return *std::min_element(values.begin(), values.end());
      }
      
    
    };
    #endif
  
  };

  /*!@brief Max element over a range
   * The functor returns a reference to the max element over the provided range
   * @author Raffi Enficiaud
   */
  template <class in_t>
  struct s_max_element_subset {
  
    typedef neighborhood_operator_traits::shiftable_operator operator_t;
  
    template <class it>
    const in_t& operator()(it beg, const it e) const {
      DEBUG_ASSERT(beg != e, "Empty neighborhood");
      return *std::max_element(beg, e);
    }
      
  };



}}



#endif /* YAYI_LOWLEVEL_MORPHOLOGY_NEIGHBOR_OPERATORS_HPP__ */
