/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */



#include "main.hpp"
#include "Yayi/core/yayiPixelProcessing/image_constant.hpp"
#include <Yayi/core/yayiImageCore/include/yayiImageCore_Impl.hpp>
#include "Yayi/core/yayiPixelProcessing/include/image_constant_T.hpp"

#include <iostream>

using namespace yayi;

class image_set_constant
{
public:

  typedef Image<yaUINT8> image_t;

  image_t im;
  

  void test_create()
  {
    BOOST_CHECKPOINT("image_set_constant::test_create");

    image_t::coordinate_type coord;

    coord[0] = 10;
    coord[1] = 20;
      
    yaRC res = im.SetSize(coord);
    BOOST_REQUIRE(res == yaRC_ok);


    res = im.AllocateImage();
    BOOST_REQUIRE_MESSAGE(res == yaRC_ok, "Allocate failed with coords : " << coord);

    BOOST_CHECK_MESSAGE(im.Size()[0] == 10, "Bad image size : " << im.Size()[0] << " != " << 10);
    BOOST_CHECK_MESSAGE(im.Size()[1] == 20, "Bad image size : " << im.Size()[1] << " != " << 20);
    BOOST_CHECK(im.IsAllocated());
      
  }
  
  void test_set_constant()
  {
    BOOST_CHECKPOINT("image_set_constant::test_set_constant");
    
    int i = 0;
    for(image_t::iterator it = im.begin_block(), ite = im.end_block(); it != ite; ++it, i++) {
      *it = i;
      BOOST_CHECK_MESSAGE(im.pixel(i) == i, "failure with *it = i at pixel " << i << " : it = " << (int)(*it));
    }
    BOOST_CHECK_MESSAGE(i == 200, "The number of points " << i << " is different from the number of pixels in the image (200)");
      
    yaRC res = Constant(&im, (yaUINT8)11);
    BOOST_CHECK_MESSAGE(res == yaRC_ok, "Return of Constant shows an error : " << "\n\t\"" << res << "\"");

    i = 0;
    for(image_t::iterator it = im.begin_block(), ite = im.end_block(); it != ite; ++it, i++) {
      BOOST_CHECK_MESSAGE(im.pixel(i) == 11, "failure with *it = i at pixel " << i << " : it = " << (int)(*it));
    }
    BOOST_CHECK_MESSAGE(i == 200, "The number of points " << i << " is different from the number of pixels in the image (200)");
      
      
  }
  
  
  static void declare_tests(test_suite*& test) {
    boost::shared_ptr<image_set_constant> instance( new image_set_constant );
    test->add( BOOST_CLASS_TEST_CASE( &image_set_constant::test_create,  instance ) );
    test->add( BOOST_CLASS_TEST_CASE( &image_set_constant::test_set_constant,  instance ) );
  }
};


void register_simple_interface_tests(test_suite*& test)
{
  image_set_constant::declare_tests(test);
}
