/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */




/*!@file
 * This file test the functions associated to structuring elements (ie. not the neighborhoods)
 *
 * @author Raffi Enficiaud
 */

#include "main.hpp"
#include "Yayi/core/yayiStructuringElement/yayiStructuringElement.hpp"
#include "Yayi/core/yayiStructuringElement/yayiRuntimeStructuringElements_predefined.hpp"
#include <iostream>

using namespace yayi;
using namespace yayi::se;


class se_predefined_basic_tests
{
public:

  void test_predefined_se()
  {
    BOOST_CHECKPOINT("test_predefined_se");
    
    BOOST_CHECK(SEHex2D.size() == 14);
    BOOST_CHECK(SEHex2D.number_of_list() == 2);
    BOOST_CHECK(SEHex2D.has_multiple_list());
    BOOST_CHECK(SEHex2D.GetType() == e_set_neighborlist);
    BOOST_CHECK(SEHex2D.GetSubType() == e_sest_neighborlist_hexa);

    BOOST_CHECK(SEHex2D.maximum_extension().first == c2D(-1,-1));
    BOOST_CHECK_MESSAGE(SEHex2D.maximum_extension().second == c2D(1,1), "Maximal extension error : " << SEHex2D.maximum_extension().second << " != " << c2D(1,1));
    
    BOOST_CHECK(SESquare2D.size() == 9);
    BOOST_CHECK(SESquare2D.number_of_list() == 1);
    BOOST_CHECK(!SESquare2D.has_multiple_list());
    BOOST_CHECK(SESquare2D.GetType() == e_set_neighborlist);
    BOOST_CHECK(SESquare2D.GetSubType() == e_sest_neighborlist_generic_single);


    BOOST_CHECK(SESquare3D.size() == 27);
    BOOST_CHECK(SESquare3D.number_of_list() == 1);
    BOOST_CHECK(!SESquare3D.has_multiple_list());
    BOOST_CHECK(SESquare3D.GetType() == e_set_neighborlist);
    BOOST_CHECK(SESquare3D.GetSubType() == e_sest_neighborlist_generic_single);
    
  }
  

  
  static void register_tests(test_suite*& test) {
    boost::shared_ptr<se_predefined_basic_tests> instance( new se_predefined_basic_tests );
    test->add( BOOST_CLASS_TEST_CASE( &se_predefined_basic_tests::test_predefined_se,  instance ) );  
  }
  
};


class se_test_basic
{
public:

  void test_interface_se()
  {
    BOOST_CHECKPOINT("test_interface_se");	
  }

  void test_clone()
  {
    BOOST_CHECKPOINT("test_predefined_se");
    
    const IStructuringElement* se_ = &SEHex2D, *se2_ = se_->Clone(), *se3_ = se_->Transpose();
    
    BOOST_CHECK(se_);
    BOOST_CHECK(se2_);
    BOOST_CHECK(se3_);
        
    delete se2_;
    delete se3_;
  }
  
  static void register_tests(test_suite*& test) {
    boost::shared_ptr<se_test_basic> instance( new se_test_basic );
    test->add( BOOST_CLASS_TEST_CASE( &se_test_basic::test_interface_se,  instance ) );  
    test->add( BOOST_CLASS_TEST_CASE( &se_test_basic::test_clone,  instance ) );  
  
  }
  
};






void register_se_test(test_suite*& test)
{
  se_predefined_basic_tests::register_tests(test);
  se_test_basic::register_tests(test);
}

